from markdown_refdocs.markdown import constant_to_markdown, create_type_link, function_to_markdown


class TestConstantToMarkdown:
    def test_no_source_code(self):
        md = constant_to_markdown({'name': 'SOME_CONSTANT', 'source_code': 'SOME_CONSTANT = "bob"'})
        assert (
            md
            == '''## SOME_CONSTANT

```python
SOME_CONSTANT = "bob"
```
'''
        )

    def test_has_source_code(self):
        md = constant_to_markdown({'name': 'SOME_CONSTANT', 'source_code': ''})
        assert (
            md
            == '''## SOME_CONSTANT
'''
        )


class TestFunctionToMarkdown:
    def test_has_docstring_no_params(self):
        md = function_to_markdown(
            {
                'name': 'some_func',
                'source_code': '',
                'source_definition': '',
                'description': 'some desc',
            }
        )
        assert (
            md
            == '''## some\\_func()

some desc
'''
        )

    def test_long_docstring(self):
        desc = """Split the input text into a prefix and suffix, according to the following patterns:
If the input string is letters followed by numbers, return them separately in a tuple.
If the input string is only letters, return a tuple of ({letters}, None).
If the input string is only numbers, return a tuple of (None, {numbers}).
Otherwise return a tuple of Nones."""
        md = function_to_markdown(
            {'name': 'some_func', 'source_code': '', 'source_definition': '', 'description': desc}
        )
        assert (
            md
            == '''## some\\_func()

Split the input text into a prefix and suffix, according to the following patterns:
If the input string is letters followed by numbers, return them separately in a tuple.
If the input string is only letters, return a tuple of ({letters}, None).
If the input string is only numbers, return a tuple of (None, {numbers}).
Otherwise return a tuple of Nones.
'''
        )


class TestCreateTypeLink:
    def test_simple_link(self):
        link = './path/to/other/#class-someclass'
        types = {'type': link}
        md = create_type_link('type', types)
        assert md == f'[type]({link})'

    def test_link_inside_list(self):
        link = './path/to/other/#class-someclass'
        types = {'type': link}
        md = create_type_link('List[type]', types)
        assert md == f'List\\[[type]({link})\\]'

    def test_linking_as_dict_target(self):
        link = './path/to/other/#class-someclass'
        types = {'type': link}
        md = create_type_link('Dict[str, type]', types)
        assert md == f'Dict\\[`str`, [type]({link})\\]'

    def test_tuple_of_links(self):
        link = './path/to/other/#class-someclass'
        types = {'type': link}
        md = create_type_link('Tuple[type, type]', types)
        assert md == f'Tuple\\[[type]({link}), [type]({link})\\]'



from typing import List

try:
    from typing import TypedDict  # type: ignore
except ImportError:
    from typing_extensions import TypedDict

Parsed: TypedDict = TypedDict('Parsed', {'name': str, 'source_code': str, 'hidden': bool})


class ParsedParameter(Parsed):
    default_value: str
    type: str
    description: str


class ParsedReturn(TypedDict):
    type: str
    description: str


class ParsedFunction(Parsed):
    """
    Result of the combination of parsing the source code and combining with the docstring of a function
    """

    parameters: List[ParsedParameter]
    returns: ParsedReturn
    note: str
    raises: List[str]
    examples: List[str]
    description: str
    is_static: bool
    is_method: bool
    is_class_method: bool
    is_getter: bool


class ParsedDocstring(TypedDict):
    description: str
    note: str
    examples: List[str]
    raises: List[str]
    returns: ParsedReturn
    parameters: List[ParsedParameter]
    attributes: List[ParsedParameter]


class ParsedVariable(Parsed):
    type: str
    description: str
    value: str


class ParsedClass(Parsed):
    functions: List[ParsedFunction]
    variables: List[ParsedVariable]
    description: str
    inherits: List[str]


class ParsedModule(Parsed):
    classes: List[ParsedClass]
    functions: List[ParsedFunction]
    variables: List[ParsedVariable]


ADMONITIONS = ['warning', 'note', 'info', 'bug', 'tip', 'question', 'failure', 'danger', 'quote']



from markdown_refdocs.links import create_relative_types_mapping, create_types_mapping
from markdown_refdocs.types import ParsedClass, ParsedModule, ParsedVariable


class TestCreateTypesMapping:
    def test_class_same_file(self) -> None:
        inputs = {
            'path/mod1': ParsedModule(
                {
                    'name': 'mod1',
                    'classes': [ParsedClass({'name': 'SomeClass'})],
                    'variables': [ParsedVariable({'name': 'SOME_VARIABLE', 'type': 'SomeClass'})],
                }
            )
        }
        mapping = create_types_mapping(inputs)
        assert mapping == {
            'SomeClass': './path/mod1/#class-someclass',
            'mod1.SomeClass': './path/mod1/#class-someclass',
        }

    def test_drop_name_clashes(self) -> None:
        inputs = {
            'path/mod1': ParsedModule(
                {
                    'name': 'mod1',
                    'classes': [ParsedClass({'name': 'SomeClass'})],
                    'variables': [ParsedVariable({'name': 'SOME_VARIABLE', 'type': 'SomeClass'})],
                }
            ),
            'path/mod2': ParsedModule(
                {
                    'name': 'mod2',
                    'classes': [ParsedClass({'name': 'SomeClass'})],
                    'variables': [ParsedVariable({'name': 'SOME_VARIABLE', 'type': 'SomeClass'})],
                }
            ),
        }
        mapping = create_types_mapping(inputs)
        assert mapping == {
            'mod1.SomeClass': './path/mod1/#class-someclass',
            'mod2.SomeClass': './path/mod2/#class-someclass',
        }

    def test_class_other_file(self) -> None:
        inputs = {
            'path/mod1': ParsedModule(
                {'name': 'mod1', 'classes': [ParsedClass({'name': 'SomeClass'})]}
            ),
            'path/mod2': ParsedModule(
                {
                    'name': 'mod2',
                    'variables': [ParsedVariable({'name': 'SOME_VARIABLE', 'type': 'SomeClass'})],
                }
            ),
        }
        mapping = create_types_mapping(inputs)
        assert mapping == {
            'SomeClass': './path/mod1/#class-someclass',
            'mod1.SomeClass': './path/mod1/#class-someclass',
        }


class TestCreateRelativeTypesMapping:
    def test_self_links(self):
        initial = {
            'type1': 'things/thing1.md/#class-firstclass',
            'type2': 'things/thing2.md/#class-firstclass',
        }
        relative = {
            'type1': '#class-firstclass',
            'type2': '../thing2/#class-firstclass',
        }
        assert create_relative_types_mapping('things/thing1.md', initial) == relative



from markdown_refdocs.parsers import parse_google_docstring


class TestParseGoogleDocstring:
    def test_nested_types(self):
        result = parse_google_docstring(
            """
Args:
    arg2 (List[Dict]): list of records
"""
        )
        assert len(result['parameters']) == 1
        assert result['parameters'][0]['type'] == 'List[Dict]'

    def test_add_extra_returns_to_description(self):
        result = parse_google_docstring(
            """
Returns:
    response: the response
    data (list, dict or None): The ids and type of resource object(s) in this relationship.
    meta (dict): meta information about the response
        total (int): total records available in the relationship
        count (int): total records returned in the response
        limit (int): the page size
        offset (int): starting point of the page
    links (dict): paging links to prev and next page, plus link to current request
"""
        )
        assert (
            result['returns']['description']
            == 'the response data (list, dict or None): The ids and type of resource object(s) in this relationship. meta (dict): meta information about the response total (int): total records available in the relationship count (int): total records returned in the response limit (int): the page size offset (int): starting point of the page links (dict): paging links to prev and next page, plus link to current request'
        )

    def test_long_docstring_with_newlines(self):
        result = parse_google_docstring(
            """
    Split the input text into a prefix and suffix, according to the following patterns:

    If the input string is letters followed by numbers, return them separately in a tuple.
    If the input string is only letters, return a tuple of ({letters}, None).
    If the input string is only numbers, return a tuple of (None, {numbers}).
    Otherwise return a tuple of Nones.
    Args:
        text (str): the input to split.
    Returns:
        tuple: The result of splitting the input string.
    Examples:
        prefix_split('A12345') == ('A', '12345')
        prefix_split('HTMCP_1') == ('HTMCP_', '1')
        prefix_split('ABC') == ('ABC', None)
        prefix_split('12345') == (None, '12345')
        prefix_split('A123B') == (None, None)"""
        )
        expected = """Split the input text into a prefix and suffix, according to the following patterns:

If the input string is letters followed by numbers, return them separately in a tuple.
If the input string is only letters, return a tuple of ({letters}, None).
If the input string is only numbers, return a tuple of (None, {numbers}).
Otherwise return a tuple of Nones."""
        assert result['description'] == expected



import re
from typing import Any, Dict

from .types import ParsedDocstring, ParsedParameter, ParsedReturn, ADMONITIONS


def left_align_block(block: str) -> str:
    lines = block.split('\n')
    if not lines:
        return block

    indent = re.match(r'(\s*).*', lines[0]).group(1)  # type: ignore
    content = '\n'.join([line[len(indent) :] for line in lines])
    return content


def parse_google_docstring(
    docstring: str, hide_undoc: bool = True, function_name=''
) -> ParsedDocstring:
    """
    parses a google-style docsting into a dictionary of the various sections
    """
    state = None
    tags = ['args', 'returns', 'raises', 'note', 'desc', 'example', 'attributes', 'warning']
    content: Dict[str, Any] = {tag: [] for tag in tags}

    docstring = (docstring if docstring else '').strip()

    # start with the description
    for line in docstring.split('\n'):
        line = line.strip()
        if not line and state is None:
            continue
        new_state = line.lower().replace(':', '') if line.endswith(':') else ''

        if new_state in tags:
            state = new_state
            if state == 'example':
                content[state].append([])
            continue

        if state is None:
            # no elements yet
            state = 'desc'
            content['desc'] = [line]
        elif state == 'desc':
            content['desc'].append(line)
        elif line:
            if state == 'example':
                content[state][-1].append(line)
            elif state in tags:
                content[state].append(line)

    result = ParsedDocstring(
        {
            'raises': [],
            'attributes': [],
            'parameters': [],
            'description': '',
            'returns': ParsedReturn({}),
            'examples': [],
        }
    )

    for tag in ['attributes', 'args']:
        parsed_args = []
        for i, arg in enumerate(content[tag]):
            try:
                name, _, arg_type, arg_desc = re.match(  # type: ignore
                    r'^(\w+)(\s+\(([^)]+)\))?:\s*(.*)$', arg
                ).groups()
                parsed_args.append(
                    ParsedParameter(
                        {
                            'name': name,
                            'type': arg_type,
                            'description': arg_desc,
                            'hidden': not arg_desc and hide_undoc,
                        }
                    )
                )
            except AttributeError:
                pass

        if tag == 'args':
            result['parameters'] = parsed_args
        else:
            result[tag] = parsed_args

    for i, line in enumerate(content['raises']):
        _, arg_type, arg_desc = re.match(r'^(([^:]+):)?\s*(.*)$', line).groups()  # type: ignore
        result['raises'].append(ParsedReturn({'type': arg_type, 'description': arg_desc}))

    for i, line in enumerate(content['returns']):
        _, arg_type, arg_desc = re.match(r'^(([^:]+):)?\s*(.*)$', line).groups()  # type: ignore
        if result['returns']:
            result['returns']['description'] += ' ' + line
        else:
            print(
                f'warning: {function_name} multiple return lines, being appended to the description'
            )
            result['returns'] = ParsedReturn({'type': arg_type, 'description': arg_desc})

    result['description'] = '\n'.join(content['desc']).strip()

    for i, example in enumerate(content['example']):
        result['examples'].append(left_align_block('\n'.join(example)))

    for admon in ADMONITIONS:
        if content.get(admon, []):
            result[admon] = content[admon]

    return result



import re
from typing import Dict, List

from .types import ParsedClass, ParsedFunction, ParsedModule, ParsedVariable, ADMONITIONS


def create_type_link(type_name: str, types_links: Dict[str, str] = {}) -> str:
    tokens = re.split(r'(\[|\]|\s|,)', str(type_name))

    linked = []
    contains_link = False
    for token in tokens:
        if token in types_links:
            contains_link = True
            linked.append(f'[{token}]({types_links[token]})')
        elif token in ['str', 'int', 'bool', 'float']:
            linked.append(f'`{token}`')
        elif token in [']', '[']:
            linked.append(f'\\{token}')
        else:
            linked.append(token)

    if not contains_link:
        return f'`{type_name}`'
    return ''.join(linked)


def argument_md(
    name: str = '',
    type: str = None,
    description: str = '',
    default_value: str = None,
    hidden: bool = False,
    types_links: Dict[str, str] = {},
    **kwargs,
) -> str:
    md = ''
    if hidden:
        return ''
    if type:
        type = create_type_link(type, types_links)

    if type and not name:
        md = f'- {type}'
    elif name:
        md = f'- {name}'
        if type:
            md = f'- {name} ({type})'

    if description:
        md += f': {description}'
    return md


def admonitions_to_markdown(parsed: Dict[str, List[str]]) -> str:
    md: List[str] = []
    for admonition in ADMONITIONS:
        if parsed.get(admonition, []):
            md.append(f'!!! {admonition}')
            for line in parsed[admonition]:
                md.append(f'\t{line}')
            md.append('')
    return '\n'.join(md)


def function_to_markdown(
    parsed: ParsedFunction, heading_level: int = 2, types_links: Dict[str, str] = {}
) -> str:
    """
    Generate a markdown string for a function definition
    """
    if parsed.get('hidden', False):
        return ''
    heading = '#' * heading_level
    name = parsed['name'].replace('_', '\\_')
    md = [f'{heading} {name}()\n']
    if parsed.get('description', ''):
        md.extend([parsed['description'], ''])
    if parsed.get('source_definition', ''):
        md.append(f'```python\n{parsed["source_definition"]}\n```\n')

    if parsed.get('parameters', ''):
        md.append('**Args**\n')
        for arg in parsed['parameters']:
            md.append(argument_md(types_links=types_links, **arg))
        md.append('')

    if parsed.get('returns', ''):
        md.append('**Returns**\n')
        md.append(argument_md(types_links=types_links, **parsed['returns']))
        md.append('')

    if parsed.get('raises', ''):
        md.append('**Raises**\n')
        for arg in parsed['raises']:
            md.append(argument_md(types_links=types_links, **arg))
        md.append('')

    if parsed.get('examples', ''):
        md.append('**Examples**\n')
        for example in parsed['examples']:
            md.append(f'```python\n{example}\n```\n')
        md.append('')

    admon_md = admonitions_to_markdown(parsed)
    if admon_md:
        md.append(admon_md)

    return '\n'.join(md)


def class_to_markdown(parsed: ParsedClass, types_links: Dict[str, str] = {}) -> str:
    if parsed.get('hidden', False):
        return ''
    md = [f'## class {parsed["name"]}', '']
    if parsed.get('inherits', False):
        md.extend(
            [
                '**inherits** '
                + ' '.join([create_type_link(t, types_links) for t in parsed['inherits']]),
                '',
            ]
        )

    if parsed.get('description', ''):
        md.extend([parsed['description'], ''])

    if parsed.get('attributes', ''):
        md.append('**Attributes**\n')
        for attr in parsed['attributes']:
            md.append(argument_md(types_links=types_links, **attr))
        md.append('')

    admon_md = admonitions_to_markdown(parsed)
    if admon_md:
        md.append(admon_md)

    if parsed.get('functions', ''):
        for func in parsed['functions']:
            md.append(function_to_markdown(func, heading_level=3, types_links=types_links))
        md.append('')
    return '\n'.join(md)


def constant_to_markdown(parsed: ParsedVariable, types_links: Dict[str, str] = {}) -> str:
    md = [f'## {parsed["name"]}', '']
    if parsed['source_code']:
        md.append(f'```python\n{parsed["source_code"]}\n```\n')

    if parsed.get('attributes', ''):
        md.append('**Attributes**\n')
        for attr in parsed['attributes']:
            md.append(argument_md(types_links=types_links, **attr))
        md.append('')

    return '\n'.join(md)


def module_to_markdown(parsed: ParsedModule, types_links: Dict[str, str] = {}) -> str:
    if parsed.get('hidden', False):
        return ''
    md = [f'# {parsed["name"]}\n']

    if parsed['description']:
        md.extend([parsed['description'], ''])

    if parsed['variables']:
        for variable in parsed['variables']:
            md.append(constant_to_markdown(variable, types_links=types_links))

    for cls in parsed['classes']:
        md.append(class_to_markdown(cls, types_links=types_links))

    for func in parsed['functions']:
        md.append(function_to_markdown(func, types_links=types_links))

    return '\n'.join(md)



import os
import sys
from unittest.mock import mock_open, patch

from markdown_refdocs.main import command_interface, parse_module_file
from markdown_refdocs.markdown import module_to_markdown
from markdown_refdocs.types import ParsedVariable


class TestParseModuleFile:
    def test_multiple_decorators(self):
        data = """
from pyramid.view import (
    view_config,
)
from pyramid.security import (
    NO_PERMISSION_REQUIRED
)

@view_config(route_name='swagger', renderer='lims:templates/swagger.mak', permission=NO_PERMISSION_REQUIRED)
@view_config(route_name='swagger.json', renderer='json', permission=NO_PERMISSION_REQUIRED)
def swagger(request):
    '''I am a docstring'''
    pass
"""
        expected = """
# simple_module

## swagger()

I am a docstring

```python
@view_config(route_name='swagger', renderer='lims:templates/swagger.mak', permission=NO_PERMISSION_REQUIRED)
@view_config(route_name='swagger.json', renderer='json', permission=NO_PERMISSION_REQUIRED)
def swagger(request):
```

**Args**

- request
"""
        with patch('builtins.open', mock_open(read_data=data)):
            md = module_to_markdown(parse_module_file('simple_module.py', ''))
            assert md.strip() == expected.strip()

    def test_simple_module(self):
        data = """
def simple_function(arg1: str, arg2: int):
    '''
    this function does stuff
    '''
    pass
"""
        expected = """# simple_module

## simple\\_function()

this function does stuff

```python
def simple_function(arg1: str, arg2: int):
```

**Args**

- arg1 (`str`)
- arg2 (`int`)
"""
        with patch('builtins.open', mock_open(read_data=data)):
            md = module_to_markdown(parse_module_file('simple_module.py', ''))
            assert md.strip() == expected.strip()

    def test_overwrite_docstring_with_annotations(self):
        data = """
def simple_function(arg1: str, arg2: int):
    '''
    this function does stuff

    Args:
        arg1 (int): this is the first argument
    '''
    pass
"""
        expected = """# simple_module

## simple\\_function()

this function does stuff

```python
def simple_function(arg1: str, arg2: int):
```

**Args**

- arg1 (`str`): this is the first argument
- arg2 (`int`)
"""
        with patch('builtins.open', mock_open(read_data=data)):
            md = module_to_markdown(parse_module_file('simple_module.py', ''))
            assert md.strip() == expected.strip()

    def test_examples_from_docstring(self):
        data = """
def simple_function(arg1: str, arg2: int):
    '''
    this function does stuff

    Args:
        arg1 (int): this is the first argument

    Example:
        >>> simple_function('1', 2)
        'something'
    '''
    pass
"""
        expected = """# simple_module

## simple\\_function()

this function does stuff

```python
def simple_function(arg1: str, arg2: int):
```

**Args**

- arg1 (`str`): this is the first argument
- arg2 (`int`)

**Examples**

```python
>>> simple_function('1', 2)
'something'
```
"""
        with patch('builtins.open', mock_open(read_data=data)):
            md = module_to_markdown(parse_module_file('simple_module.py', ''))
            assert md.strip() == expected.strip()

    def test_returns_from_type_annotation(self):
        data = """
def simple_function(arg1: str, arg2: int) -> str:
    '''
    this function does stuff

    Args:
        arg1 (int): this is the first argument
    '''
    pass
"""
        expected = """# simple_module

## simple\\_function()

this function does stuff

```python
def simple_function(arg1: str, arg2: int) -> str:
```

**Args**

- arg1 (`str`): this is the first argument
- arg2 (`int`)

**Returns**

- `str`
"""
        with patch('builtins.open', mock_open(read_data=data)):
            md = module_to_markdown(parse_module_file('simple_module.py', ''))
            assert md.strip() == expected.strip()

    def test_parses_raise(self):
        data = """
def simple_function(arg1: str, arg2: int):
    '''
    this function does stuff

    Args:
        arg1 (int): this is the first argument

    Raises:
        NotImplementedError: stuff
    '''
    pass
"""
        expected = """# simple_module

## simple\\_function()

this function does stuff

```python
def simple_function(arg1: str, arg2: int):
```

**Args**

- arg1 (`str`): this is the first argument
- arg2 (`int`)

**Raises**

- `NotImplementedError`: stuff
"""
        with patch('builtins.open', mock_open(read_data=data)):
            md = module_to_markdown(parse_module_file('simple_module.py', ''))
            assert md.strip() == expected.strip()

    def test_multiline_function_def(self):
        data = """
def simple_function(
    arg1: str,
    arg2: int,
):
    '''
    this function does stuff

    Args:
        arg1 (int): this is the first argument
    '''
    pass
"""
        expected = """# simple_module

## simple\\_function()

this function does stuff

```python
def simple_function(
    arg1: str,
    arg2: int,
):
```

**Args**

- arg1 (`str`): this is the first argument
- arg2 (`int`)
"""
        with patch('builtins.open', mock_open(read_data=data)):
            md = module_to_markdown(parse_module_file('simple_module.py', ''))
            assert md.strip() == expected.strip()

    def test_class_desc(self):
        data = """
class SomeClass:
    '''
    say something

    Attributes:
        attr1 (int): an thing on a class instance
    '''
    pass
"""
        expected_md = """# simple_module

## class SomeClass

say something

**Attributes**

- attr1 (`int`): an thing on a class instance
"""
        expected_parse = {
            'name': 'simple_module',
            'hidden': False,
            'functions': [],
            'description': '',
            'variables': [],
            'classes': [
                {
                    'name': 'SomeClass',
                    'description': 'say something',
                    'inherits': [],
                    'hidden': False,
                    'functions': [],
                    'examples': [],
                    'attributes': [
                        {
                            'hidden': False,
                            'name': 'attr1',
                            'type': 'int',
                            'description': 'an thing on a class instance',
                        }
                    ],
                }
            ],
        }
        with patch('builtins.open', mock_open(read_data=data)):
            parsed = parse_module_file('simple_module.py', '')
            assert parsed == expected_parse
            md = module_to_markdown(parsed)
            assert md.strip() == expected_md.strip()

    def test_class_methods(self):
        data = """
class SomeClass:
    def some_method_on_a_class(self):
        '''
        I am a method
        '''
        pass
"""
        expected = """# simple_module

## class SomeClass

### SomeClass.some\\_method\\_on\\_a\\_class()

I am a method

```python
def some_method_on_a_class(self):
```
"""
        with patch('builtins.open', mock_open(read_data=data)):
            md = module_to_markdown(parse_module_file('simple_module.py', ''))
            assert md.strip() == expected.strip()

    def test_pass_class_args_to_construcor(self):
        data = """
class SomeClass:
    '''
    Args:
        arg1: I am in the wrong place but can be moved
    '''
    def __init__(self, arg1):
        '''
        I am a method
        '''
        pass
"""
        expected = """# simple_module

## class SomeClass

### SomeClass.\\_\\_init\\_\\_()

I am a method

```python
def __init__(self, arg1):
```

**Args**

- arg1: I am in the wrong place but can be moved
"""
        with patch('builtins.open', mock_open(read_data=data)):

            md = module_to_markdown(parse_module_file('simple_module.py', ''))
            assert md.strip() == expected.strip()

    def test_module_constant(self):
        data = """
CONSTANT_THING = 'some constant thing'
"""
        expected = """# simple_module

## CONSTANT_THING

```python
CONSTANT_THING = 'some constant thing'
```
"""
        with patch('builtins.open', mock_open(read_data=data)):

            md = module_to_markdown(parse_module_file('simple_module.py', '', hide_undoc=False))
            assert md.strip() == expected.strip()

    def test_module_constant_pulled_when_any_docstring(self):
        data = """
'''
this is a docstring for the module
'''

CONSTANT_THING = 'some constant thing'
"""
        expected = """# simple_module

this is a docstring for the module

## CONSTANT_THING

```python
CONSTANT_THING = 'some constant thing'
```
"""
        with patch('builtins.open', mock_open(read_data=data)):

            md = module_to_markdown(parse_module_file('simple_module.py', ''))
            assert md.strip() == expected.strip()

    def test_nested_return_type_annotation(self):
        data = """
def some_function() -> List[Dict]:
    pass
"""
        expected = """# simple_module

## some\\_function()

```python
def some_function() -> List[Dict]:
```

**Returns**

- `List[Dict]`
"""
        with patch('builtins.open', mock_open(read_data=data)):

            md = module_to_markdown(parse_module_file('simple_module.py', '', hide_undoc=False))
            assert md.strip() == expected.strip()

    def test_multiline_module_constant(self):
        data = """
CONSTANT_THING = [
    'some',
    'constant thing'
]
"""
        expected = """# simple_module

## CONSTANT_THING

```python
CONSTANT_THING = [
    'some',
    'constant thing'
]
```
"""
        with patch('builtins.open', mock_open(read_data=data)):

            md = module_to_markdown(parse_module_file('simple_module.py', '', hide_undoc=False))
            assert md.strip() == expected.strip()

    def test_complex_types(self):
        data = """
from typing import List, Dict, Tuple

def some_function() -> Tuple[List, Dict[str,int]]:
    pass
"""
        expected = """# simple_module

## some\\_function()

```python
def some_function() -> Tuple[List, Dict[str,int]]:
```

**Returns**

- `Tuple[List, Dict[str, int]]`
"""
        with patch('builtins.open', mock_open(read_data=data)):

            md = module_to_markdown(parse_module_file('simple_module.py', '', hide_undoc=False))
            assert md.strip() == expected.strip()

    def test_namespaced_types(self):
        data = """
from typing import List, Dict, Tuple
import ast

def some_function(arg1: ast.AST):
    pass
"""
        expected = """# simple_module

## some\\_function()

```python
def some_function(arg1: ast.AST):
```

**Args**

- arg1 (`ast.AST`)
"""
        with patch('builtins.open', mock_open(read_data=data)):

            md = module_to_markdown(parse_module_file('simple_module.py', '', hide_undoc=False))
            assert md.strip() == expected.strip()

    def test_callable(self):
        data = """
class ResourceFieldDef(_ResourceFieldDefReq, total=False):
    update: Callable[[Any, Union[int, str]], None]
    deprecated: bool
"""
        expected = """# simple_module

## class ResourceFieldDef

**inherits** `_ResourceFieldDefReq`

**Attributes**

- update (`Callable[[Any, Union[int, str]], None]`)
- deprecated (`bool`)
"""
        with patch('builtins.open', mock_open(read_data=data)):
            md = module_to_markdown(parse_module_file('simple_module.py', '', hide_undoc=False))
            assert md.strip() == expected.strip()

    def test_typed_dict(self):
        data = """
from typings import TypedDict

class SomeType(TypedDict):
    name: str
    parent: str
    grandparent: str
"""
        expected = """# simple_module

## class SomeType

**inherits** `TypedDict`

**Attributes**

- name (`str`)
- parent (`str`)
- grandparent (`str`)
"""
        expect_parsed = {
            'name': 'simple_module',
            'hidden': False,
            'variables': [],
            'functions': [],
            'description': '',
            'classes': [
                {
                    'name': 'SomeType',
                    'inherits': ['TypedDict'],
                    'description': '',
                    'hidden': False,
                    'examples': [],
                    'attributes': [
                        {'name': 'name', 'type': 'str', 'source_code': 'name: str'},
                        {'name': 'parent', 'type': 'str', 'source_code': 'parent: str'},
                        {'name': 'grandparent', 'type': 'str', 'source_code': 'grandparent: str',},
                    ],
                    'functions': [],
                }
            ],
        }

        with patch('builtins.open', mock_open(read_data=data)):
            parsed = parse_module_file('simple_module.py', '', hide_undoc=False)
            assert parsed == expect_parsed
            md = module_to_markdown(parsed)
            assert md.strip() == expected.strip()

    def test_typed_dict_with_docstring(self):
        data = """
from typings import TypedDict

class SomeType(TypedDict):
    '''
    Attributes:
        parent: the name of the parent
    '''
    name: str
    parent: str
    grandparent: str
"""
        expected = """# simple_module

## class SomeType

**inherits** `TypedDict`

**Attributes**

- name (`str`)
- parent (`str`): the name of the parent
- grandparent (`str`)
"""
        expect_parsed = {
            'name': 'simple_module',
            'hidden': False,
            'variables': [],
            'functions': [],
            'description': '',
            'classes': [
                {
                    'name': 'SomeType',
                    'description': '',
                    'inherits': ['TypedDict'],
                    'examples': [],
                    'hidden': False,
                    'functions': [],
                    'attributes': [
                        {'name': 'name', 'type': 'str', 'source_code': 'name: str'},
                        {
                            'name': 'parent',
                            'type': 'str',
                            'source_code': 'parent: str',
                            'description': 'the name of the parent',
                            'hidden': False,
                        },
                        {'name': 'grandparent', 'type': 'str', 'source_code': 'grandparent: str'},
                    ],
                }
            ],
        }

        with patch('builtins.open', mock_open(read_data=data)):
            parsed = parse_module_file('simple_module.py', '', hide_undoc=False)
            assert parsed == expect_parsed
            md = module_to_markdown(parsed)
            assert md.strip() == expected.strip()

    def test_alternate_typeddict_notation(self):
        data = """try:
    from typing import TypedDict  # type: ignore
except ImportError:
    from typing_extensions import TypedDict

Parsed: TypedDict = TypedDict('Parsed', {'name': str, 'source_code': str, 'hidden': bool})"""
        expect_parsed = [
            ParsedVariable(
                {
                    'type': 'TypedDict',
                    'name': 'Parsed',
                    'source_code': """Parsed: TypedDict = TypedDict('Parsed', {'name': str, 'source_code': str, 'hidden': bool})""",
                    'attributes': [
                        {'name': 'name', 'type': 'str'},
                        {'name': 'source_code', 'type': 'str'},
                        {'name': 'hidden', 'type': 'bool'},
                    ],
                }
            )
        ]

        with patch('builtins.open', mock_open(read_data=data)):
            parsed = parse_module_file('simple_module.py', '', hide_undoc=False)
            assert parsed['variables'] == expect_parsed

    def test_multiline_docstring_desc(self):
        data = """
def prefix_split(text):
    '''
    Split the input text into a prefix and suffix, according to the following patterns:

    If the input string is letters followed by numbers, return them separately in a tuple.
    If the input string is only letters, return a tuple of ({letters}, None).
    If the input string is only numbers, return a tuple of (None, {numbers}).

    Otherwise return a tuple of Nones.

    Args:
        text (str): the input to split.
    '''
    pass
"""
        expected = """# simple_module

## prefix\\_split()

Split the input text into a prefix and suffix, according to the following patterns:

If the input string is letters followed by numbers, return them separately in a tuple.
If the input string is only letters, return a tuple of ({letters}, None).
If the input string is only numbers, return a tuple of (None, {numbers}).

Otherwise return a tuple of Nones.

```python
def prefix_split(text):
```

**Args**

- text (`str`): the input to split.
"""
        with patch('builtins.open', mock_open(read_data=data)):
            parsed = parse_module_file('simple_module.py', '', hide_undoc=False)
            md = module_to_markdown(parsed)
            assert md.strip() == expected.strip()


class TestCommandInterface:
    def test_package_path(self, tmpdir):
        path = os.path.join(os.path.dirname(__file__), '../markdown_refdocs')
        with patch.object(
            sys,
            'argv',
            ['', path, '-o', tmpdir.dirname, '--show_undoc', '--show_undoc_args', '--show_private'],
        ):
            command_interface()

        modules = [
            os.path.join(tmpdir.dirname, 'markdown_refdocs', f'{m}.md')
            for m in ['main', 'markdown', 'parsers', '__init__']
        ]

        for module in modules:
            assert os.path.exists(module)



import argparse
import ast
import os
from typing import Dict, List, Optional, Tuple, Union, cast

from .links import create_relative_types_mapping, create_types_mapping
from .markdown import module_to_markdown
from .parsers import left_align_block, parse_google_docstring
from .types import (
    ParsedClass,
    ParsedDocstring,
    ParsedFunction,
    ParsedModule,
    ParsedParameter,
    ParsedReturn,
    ParsedVariable,
)


def get_by_name(name: str, list_to_search: List[Dict]) -> Optional[Dict]:
    for item in list_to_search:
        if item['name'] == name:
            return item
    return None


def get_lines_covered(node: ast.AST) -> Tuple[int, int]:
    lines = []
    if hasattr(node, 'lineno'):
        lines.append(node.lineno)
    for n in ast.walk(node):
        if hasattr(n, 'lineno'):
            lines.append(n.lineno)
    return min(lines), max(lines)


class ModuleAnalyzer(ast.NodeVisitor):
    """
    Parse a python module into reference docs using the source code and docstrings
    """

    def __init__(
        self,
        filename: str,
        prefix: str = '',
        hide_private: bool = True,
        hide_undoc: bool = True,
        hide_undoc_args: bool = True,
        namespace_headers: bool = False,
    ):
        print('processing module', filename)
        self.name = (
            filename.replace('.py', '').replace('/', '.')[len(prefix) :].replace('.__init__', '')
        )
        self.hide_private = hide_private
        self.hide_undoc = hide_undoc
        self.hide_undoc_args = hide_undoc_args
        self.namespace_headers = namespace_headers

        with open(filename, "r") as source:
            self.content = source.read()
            self.lines = self.content.split('\n')

    def get_qualified_name(self, node: ast.AST, name: str) -> Optional[str]:
        parents = []
        if not node.parent:
            return None
        parent = node.parent
        while True:
            if not self.namespace_headers and isinstance(parent, ast.Module):
                break
            try:
                parents.append(parent.name)
                parent = parent.parent
            except Exception:
                break
        parents.reverse()
        return '.'.join(parents + [name])

    def get_source_segment(self, node: ast.AST, expected_end_char: str = None) -> str:
        # only builtin  in py3.8+ so re-implemented here
        start, end = get_lines_covered(node)
        content = '\n'.join(self.lines[start - 1 : end])
        if expected_end_char and not content.strip().endswith(expected_end_char):
            end += 1
            content = '\n'.join(self.lines[start - 1 : end])
        return content

    def get_function_def_segment(self, node: ast.FunctionDef) -> str:
        """
        Get the source code lines covering the function defintion
        """
        try:
            start, end = get_lines_covered(node.args)
        except ValueError:
            start = end = node.lineno

        start = min(node.lineno, start) - 1
        content = '\n'.join(self.lines[start:end])

        if not content.strip().endswith(':'):
            end += 1
            content = '\n'.join(self.lines[start:end])

        start_shift = 0
        for backstep in range(1, len(node.decorator_list) + 1):
            if start - backstep < 0:
                break
            if not self.lines[start - backstep].strip().startswith('@'):
                break
            start_shift = backstep
        start -= start_shift
        content = '\n'.join(self.lines[start:end])

        return left_align_block(content)

    def visit_ClassDef(self, node: ast.ClassDef) -> ParsedClass:
        """convert a class into markdown"""
        result = ParsedClass(
            {
                'name': self.get_qualified_name(node, node.name),
                'description': '',
                'attributes': [],
                'functions': [],
                'inherits': [self.visit(b) for b in node.bases],
                'hidden': False,
            }
        )
        doc = parse_google_docstring(
            ast.get_docstring(node), self.hide_undoc_args, self.get_qualified_name(node, node.name)
        )
        result.update(
            {d: doc[d] for d in doc if d not in ['parameters', 'raises', 'returns', 'attributes']}
        )

        for elem in node.body:
            subnode = self.visit(elem)
            if isinstance(elem, ast.FunctionDef) and subnode:
                result['functions'].append(subnode)
            if isinstance(elem, ast.AnnAssign):
                exists = get_by_name(subnode['name'], doc['attributes']) or {}
                exists.update(subnode)
                result['attributes'].append(exists)

        # add any attribute notes from the docstring not annotated
        for attr in doc['attributes']:
            not_exists = not get_by_name(attr['name'], result['attributes'])
            if not_exists:
                result['attributes'].append(attr)
        if self.hide_undoc and not result['functions'] and not result['attributes']:
            result['hidden'] = True
        return result

    def visit_FunctionDef(self, node: ast.FunctionDef) -> ParsedFunction:
        """convert a function into markdown"""
        decorators = [self.visit(d) for d in node.decorator_list]
        result = ParsedFunction(
            {
                'name': self.get_qualified_name(node, node.name),
                'hidden': False,
                'is_class_method': 'classmethod' in decorators,
                'is_static': 'staticmethod' in decorators,
                'is_getter': 'property' in decorators,
                'returns': ParsedReturn({}),
                'parameters': [],
                'raises': [],
                'examples': [],
                'description': '',
                'source_definition': self.get_function_def_segment(node),
                'source_code': self.get_source_segment(node),
            }
        )

        diff = len(node.args.args) - len(node.args.defaults)
        defaults = list(node.args.defaults)
        class_parent = isinstance(node.parent, ast.ClassDef)
        result['is_method'] = (
            class_parent
            and not result['is_static']
            and not result['is_getter']
            and not result['is_class_method']
        )

        doc = parse_google_docstring(
            ast.get_docstring(node), self.hide_undoc_args, self.get_qualified_name(node, node.name)
        )
        result.update({d: doc[d] for d in doc if d != 'parameters'})

        class_doc = ParsedDocstring({})

        if class_parent and node.name == '__init__':
            class_doc = parse_google_docstring(
                ast.get_docstring(node.parent),
                self.hide_undoc_args,
                self.get_qualified_name(node, node.name),
            )

        if self.hide_private and node.name.startswith('_') and node.name != '__init__':
            result['hidden'] = True

        if (
            not doc['description']
            and self.hide_undoc
            and (not class_doc or not class_doc['description'])
        ):
            result['hidden'] = True

        # mix the python built-in annotations with the docstring ones
        result['returns'].update(doc['returns'])
        parsed_return = self.visit(node.returns)
        if parsed_return:
            result['returns']['type'] = parsed_return

        for index, arg in enumerate(node.args.args):
            arg_name = arg.arg
            defn = ParsedParameter({'name': arg_name})

            if index == 0:
                if arg_name == 'self' and result['is_method']:
                    continue
                if arg_name == 'cls' and result['is_class_method']:
                    continue
            # update param from docstrings
            function_arg_doc = get_by_name(arg_name, doc.get('parameters', []))
            class_arg_doc = get_by_name(arg_name, class_doc.get('parameters', []))
            if class_arg_doc:
                defn.update(class_arg_doc)
            if function_arg_doc:
                defn.update(function_arg_doc)

            if index >= diff:
                defn['default_value'] = self.visit(defaults[index - diff])
            if arg.annotation:
                defn['type'] = self.visit(arg.annotation)

            result['parameters'].append(defn)

        return result

    def visit_Module(self, node: ast.Module) -> ParsedModule:
        """
        Convert a module into markdown
        """
        classes: List[ParsedClass] = []
        functions: List[ParsedFunction] = []
        constants: List[ParsedVariable] = []
        result = ParsedModule(
            {
                'name': self.name,
                'variables': constants,
                'functions': functions,
                'classes': classes,
                'hidden': False,
                'description': ast.get_docstring(node) or '',
            }
        )
        node.name = self.name

        # register parents
        for subnode in ast.walk(node):
            for child in ast.iter_child_nodes(subnode):
                child.parent = subnode

        for elem in node.body:
            subnode = self.visit(elem)
            if not subnode:
                continue
            if isinstance(elem, ast.ClassDef):
                classes.append(cast(ParsedClass, subnode))
            elif isinstance(elem, ast.FunctionDef):
                functions.append(cast(ParsedFunction, subnode))
            elif isinstance(elem, ast.Assign):
                constants.extend(cast(ParsedVariable, subnode))
            elif isinstance(elem, ast.AnnAssign):
                constants.append(cast(ParsedVariable, subnode))

        module_docstring = ast.get_docstring(node)
        if not classes and not functions and not module_docstring and self.hide_undoc:
            result['hidden'] = True

        return result

    def visit_Name(self, node: ast.Name) -> str:
        return node.id

    def visit_NameConstant(self, node: ast.NameConstant) -> str:
        return node.value

    def visit_Str(self, node: ast.Str) -> str:
        return node.s

    def visit_NoneType(self, node: None) -> None:
        return node

    def visit_Tuple(self, node: ast.Tuple) -> str:
        content = ', '.join([str(self.visit(e)) for e in node.elts])
        return content

    def visit_List(self, node: ast.List) -> str:
        content = ', '.join([str(self.visit(e)) for e in node.elts])
        return f'[{content}]'

    def visit_Subscript(self, node: ast.Subscript) -> str:
        inner = self.visit(node.slice.value)
        return f'{self.visit(node.value)}[{inner}]'

    def visit_Assign(self, node: ast.Assign) -> List[ParsedVariable]:
        expected_end_char = None

        if isinstance(node.value, ast.List):
            expected_end_char = ']'
        elif isinstance(node.value, ast.Call):
            expected_end_char = ')'

        result: List[ParsedVariable] = []
        for target in node.targets:
            result.append(
                ParsedVariable(
                    {
                        'name': self.visit(target),
                        'source_code': left_align_block(
                            self.get_source_segment(node, expected_end_char)
                        ),
                    }
                )
            )
        return result

    def visit_AnnAssign(self, node: ast.AnnAssign) -> List[Union[ParsedVariable, ParsedClass]]:
        expected_end_char = None

        if isinstance(node.value, ast.List):
            expected_end_char = ']'
        elif isinstance(node.value, ast.Call):
            expected_end_char = ')'

        if self.visit(node.annotation) == 'TypedDict':
            return ParsedClass(
                {
                    'name': self.visit(node.target),
                    'source_code': left_align_block(
                        self.get_source_segment(node, expected_end_char)
                    ),
                    'attributes': self.visit(node.value),
                    'type': 'TypedDict',
                }
            )

        return ParsedVariable(
            {
                'name': self.visit(node.target),
                'source_code': left_align_block(self.get_source_segment(node, expected_end_char)),
                'type': self.visit(node.annotation),
            }
        )

    def visit_Attribute(self, node: ast.Attribute) -> str:
        name = self.visit(node.value)

        if node.attr:
            return f'{name}.{node.attr}'
        return name

    def visit_Call(self, node: ast.Call) -> List[ParsedVariable]:
        name = self.visit(node.func)
        if name == 'TypedDict':
            attributes = node.args[1]
            keys = [self.visit(k) for k in attributes.keys]
            values = [self.visit(v) for v in attributes.values]
            return [ParsedVariable({'name': k, 'type': v}) for (k, v) in zip(keys, values)]
        return []


def parse_module_file(
    filename: str,
    prefix: str = '',
    hide_private: bool = True,
    hide_undoc: bool = True,
    hide_undoc_args: bool = True,
    namespace_headers: bool = False,
) -> ParsedModule:
    """
    convert a module into markdown

    Args:
        filename: the path to the file to be read
        prefix: the portion of the path that is not part of the package
        hide_private: hide privated functions, do not document (does not apply to __init__)
        hide_undoc: exclude undocumented functions (no docstring)
        hide_undoc_args: do not list arguments with neither type nor description

    Returns:
        the markdown string for this module
    """

    analyzer = ModuleAnalyzer(
        filename,
        prefix,
        hide_private=hide_private,
        hide_undoc=hide_undoc,
        hide_undoc_args=hide_undoc_args,
        namespace_headers=namespace_headers,
    )
    tree = ast.parse(analyzer.content)
    content = analyzer.visit(tree)
    return content


def extract_to_markdown(
    paths: List[str],
    output_dir: str,
    hide_private: bool = True,
    hide_undoc: bool = True,
    hide_undoc_args: bool = True,
    namespace_headers: bool = False,
    link: bool = False,
) -> None:

    for path in paths:
        if path.endswith('/'):
            path = path[:-1]

        package = os.path.basename(path)
        prefix = path[0 : len(path) - len(package)]
        files = []
        if os.path.isfile(path):
            files.append(path)
        else:
            for root, dirs, walkfiles in os.walk(path):
                files.extend([os.path.join(root, w) for w in walkfiles if w.endswith('.py')])

        modules: Dict[str, ParsedModule] = {}
        type_mapping: Dict[str, str] = {}

        for filename in files:
            parsed = parse_module_file(
                filename,
                prefix,
                hide_private=hide_private,
                hide_undoc=hide_undoc,
                hide_undoc_args=hide_undoc_args,
                namespace_headers=namespace_headers,
            )
            module_filename = filename[len(prefix) :].replace('.py', '.md')

            modules[module_filename] = parsed

        if link:
            type_mapping = create_types_mapping(modules)

        for module_filename, parsed in modules.items():
            if parsed.get('hidden', False):
                continue
            module_file_output = os.path.join(output_dir, module_filename)
            dirname = os.path.dirname(module_file_output)
            os.makedirs(dirname, exist_ok=True)

            print('writing:', module_file_output)
            with open(module_file_output, 'w') as fh:
                relative_mapping = create_relative_types_mapping(module_filename, type_mapping)
                fh.write(module_to_markdown(parsed, relative_mapping))


def command_interface() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--show_private',
        default=False,
        action='store_true',
        help='show private functions in output (__init__ is not considered private is is shown by default)',
    )
    parser.add_argument(
        '--show_undoc',
        default=False,
        action='store_true',
        help='show undocumented functions (no docstring)',
    )
    parser.add_argument(
        '--show_undoc_args',
        default=False,
        action='store_true',
        help='show/list function arguments with neither type nor description',
    )
    parser.add_argument('-o', '--output_dir', help='The output directory', required=True)
    parser.add_argument(
        'inputs', nargs='+', help='path(s) to python package directories to pull docstrings from'
    )
    parser.add_argument(
        '--namespace_headers',
        default=False,
        action='store_true',
        help='If true then the package/module prefix will be prepended to function/class names. Ex. pacakge_name.module_name.function_name instead of just function_name',
    )
    parser.add_argument(
        '--link',
        default=False,
        action='store_true',
        help='Base URL to use for creating internal links',
    )
    args = parser.parse_args()
    extract_to_markdown(
        args.inputs,
        args.output_dir,
        link=args.link,
        hide_private=not args.show_private,
        hide_undoc=not args.show_undoc,
        hide_undoc_args=not args.show_undoc_args,
        namespace_headers=args.namespace_headers,
    )



import os
import re
from typing import Dict, Optional

from .types import ParsedModule


def create_types_mapping(modules: Dict[str, ParsedModule]) -> Dict[str, str]:
    """
    Creates mapping of type name to links

    Note:
        This only links within a package
    """
    simple_mapping: Dict[str, Optional[str]] = {}
    qualified_mapping: Dict[str, Optional[str]] = {}

    def add_to_mappings(module: ParsedModule, short_name: str, prefix: str = 'class-'):
        qualified_name = f'{module["name"]}.{short_name}'
        url = f'./{path}/#{prefix}{short_name.lower()}'

        if short_name in simple_mapping:
            simple_mapping[short_name] = None  # drop name clashes
        else:
            simple_mapping[short_name] = url

        if qualified_name in qualified_mapping:
            qualified_mapping[qualified_name] = None  # drop name clashes
        else:
            qualified_mapping[qualified_name] = url

    for path, module in modules.items():
        for var in module.get('variables', []):
            if re.match(r'^[A-Z][a-z]', var['name']):
                add_to_mappings(module, var['name'], '')

        for cls in module.get('classes', []):
            add_to_mappings(module, cls['name'])

    simple_mapping.update(qualified_mapping)
    return {k: v for (k, v) in simple_mapping.items() if v is not None}


def create_relative_types_mapping(current_file: str, types_mapping: Dict[str, str]):
    relative_mapping: Dict[str, str] = {}

    for linked_type, path_link in types_mapping.items():
        original_path, hash_location = path_link.split('#')
        relative_path = re.sub(
            r'\.md$', '', os.path.join(os.path.relpath(original_path, current_file))
        )
        if relative_path == '.':
            relative_mapping[linked_type] = f'#{hash_location}'
        else:
            relative_mapping[linked_type] = f'{relative_path}/#{hash_location}'
    return relative_mapping



from setuptools import find_packages, setup

# Dependencies required to use your package
INSTALL_REQS = ['typing_extensions>=3.7.4.2']

# Dependencies required only for running tests
TEST_REQS = ['pytest', 'pytest-runner', 'pytest-cov']

# Dependencies required for deploying to an index server
DEPLOYMENT_REQS = ['twine', 'wheel']

DOC_REQS = ['mkdocs', 'mkdocs-material']

DEV_REQS = (
    TEST_REQS + DEPLOYMENT_REQS + DOC_REQS + ['black', 'flake8', 'flake8-annotations', 'mypy']
)

long_description = ''

try:
    import os

    with open(os.path.join(os.path.dirname(__file__), 'README.md'), 'r') as fh:
        long_description = fh.read()
except Exception as err:
    print(err)
    pass

setup(
    name='markdown_refdocs',
    version='1.3.0',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=INSTALL_REQS,
    extras_require={
        'dev': DEV_REQS,
        'deploy': DEPLOYMENT_REQS,
        'test': TEST_REQS,
        'docs': DOC_REQS,
    },
    package_data={'markdown_refdocs': ['py.typed']},
    python_requires='>=3.6',
    author_email='caralynreisle@gmail.com',
    author='Caralyn Reisle',
    dependency_links=[],
    test_suite='tests',
    tests_require=TEST_REQS,
    entry_points={
        'console_scripts': ['markdown_refdocs = markdown_refdocs.main:command_interface']
    },
    url='https://github.com/creisle/markdown_refdocs',
)
